@@ -35,7 +35,7 @@ fn main() -> ! {
 
     let clock_cfg = Clocks::default();
 
-    clock_cfg.setup(&mut dp.RCC, &mut dp.FLASH).unwrap();
+    clock_cfg.setup().unwrap();
 
     // Configure pins for Spi
     let _sck = Pin::new(Port::A, 5, PinMode::Alt(5));
