@@ -554,11 +554,14 @@ impl Clocks {
     /// are invalid.
     /// Use the STM32CubeIDE Clock Configuration tab to help identify valid configs.
     /// Use the `default()` implementation as a safe baseline.
-    pub fn setup(&self, rcc: &mut RCC, flash: &mut FLASH) -> Result<(), SpeedError> {
+    pub fn setup(&self) -> Result<(), SpeedError> {
         if let Err(e) = self.validate_speeds() {
             return Err(e);
         }
 
+        let rcc = unsafe { &(*RCC::ptr()) };
+        let flash = unsafe { &(*FLASH::ptr()) };
+
         // Enable and reset System Configuration Controller, ie for interrupts.
         // todo: Is this the right module to do this in?
         #[cfg(not(any(feature = "wb", feature = "wl")))] // todo: Do interrupts work without enabling syscfg on wb, which
@@ -953,7 +956,9 @@ impl Clocks {
 
     /// Re-select input source; used after Stop and Standby modes, where the system reverts
     /// to MSI or HSI after wake.
-    pub fn reselect_input(&self, rcc: &mut RCC) {
+    pub fn reselect_input(&self) {
+        let rcc = unsafe { &(*RCC::ptr()) };
+
         // Re-select the input source; useful for changing input source, or reverting
         // from stop or standby mode. This assumes we're on a clean init,
         // or waking up from stop mode etc.
@@ -1073,9 +1078,11 @@ impl Clocks {
     #[cfg(any(feature = "l4", feature = "l5"))]
     /// Use this to change the MSI speed. Run this only if your clock source is MSI.
     /// Ends in a state with MSI on at the new speed, and HSI off.
-    pub fn change_msi_speed(&mut self, range: MsiRange, rcc: &mut RCC) {
+    pub fn change_msi_speed(&mut self, range: MsiRange) {
         // todo: Calibrate MSI with LSE / HSE(?) if avail?
 
+        let rcc = unsafe { &(*RCC::ptr()) };
+
         match self.input_src {
             InputSrc::Msi(_) => (),
             _ => panic!("Only change MSI speed using this function if MSI is the input source."),
@@ -1099,7 +1106,9 @@ impl Clocks {
     /// source or PLL source. You may need to re-run this after exiting `stop` mode. Only works for USB
     /// if you have an LSE connected.
     /// Note: MSIPLLEN must be enabled after LSE is enabled. So, run this function after RCC clock setup.
-    pub fn enable_msi_48(&self, rcc: &mut RCC) {
+    pub fn enable_msi_48(&self) {
+        let rcc = unsafe { &(*RCC::ptr()) };
+
         if let InputSrc::Msi(_) = self.input_src {
             panic!(
                 "Only use this function to set up MSI as 48MHz oscillator\
@@ -1169,10 +1178,11 @@ impl Clocks {
     /// in a different context. eg:
     /// ```
     /// if !clock_cfg.pll_is_enabled() {
-    ///     clock_cfg.reselect_input(&mut dp.RCC);
+    ///     clock_cfg.reselect_input();
     ///}
     ///```
-    pub fn pll_is_enabled(&self, rcc: &mut RCC) -> bool {
+    pub fn pll_is_enabled(&self) -> bool {
+        let rcc = unsafe { &(*RCC::ptr()) };
         rcc.cr.read().pllon().bit_is_set()
     }
 
@@ -1223,12 +1233,17 @@ impl Clocks {
 
     cfg_if! {
         if #[cfg(feature = "g0")] {
+            // On G0, a single APB prescaler is used for both APB1 and APB2.
             pub fn apb2(&self) -> u32 {
-                unimplemented!("No apb2 on G0");
+                self.hclk() / self.apb1_prescaler.value() as u32
             }
 
             pub fn apb2_timer(&self) -> u32 {
-                unimplemented!("No apb2 on G0");
+                if let ApbPrescaler::Div1 = self.apb1_prescaler {
+                    self.apb2()
+                } else {
+                    self.apb2() * 2
+                }
             }
         } else {
             pub fn apb2(&self) -> u32 {
@@ -1364,7 +1379,10 @@ impl Default for Clocks {
 /// external signal on CRS_SYNC pin or generated by user software. For faster lock-in during
 /// startup it is also possible to combine automatic trimming with manual trimming action."
 /// Note: This is for HSI48 only. Note that the HSI will turn off after entering Stop or Standby.
-pub fn enable_crs(sync_src: CrsSyncSrc, crs: &mut pac::CRS, rcc: &mut RCC) {
+pub fn enable_crs(sync_src: CrsSyncSrc) {
+    let crs = unsafe { &(*pac::CRS::ptr()) };
+    let rcc = unsafe { &(*RCC::ptr()) };
+
     // todo: CRSEN missing on l4x5 pac: https://github.com/stm32-rs/stm32-rs/issues/572
     cfg_if! {
         if #[cfg(feature = "l4x5")] {
