@@ -28,18 +28,19 @@ use crate::pac::dma as dma_p;
 ))]
 use crate::pac::dma1 as dma_p;
 
-#[cfg(not(any(feature = "f4", feature = "l5")))]
+#[cfg(not(feature = "l5"))]
 use crate::dma::{self, ChannelCfg, Dma, DmaChannel};
 
 #[cfg(any(feature = "f3", feature = "l4"))]
 use crate::dma::DmaInput;
 
+// todo: Get rid of this macro.
 macro_rules! busy_wait {
-    ($regs:expr, $flag:ident, $variant:ident) => {
+    ($regs:expr, $flag:ident) => {
         loop {
             let isr = $regs.isr.read();
 
-            if isr.$flag().$variant() {
+            if isr.$flag().bit_is_set() {
                 break;
             } else if isr.berr().bit_is_set() {
                 $regs.icr.write(|w| w.berrcf().set_bit());
@@ -84,14 +85,6 @@ pub enum Error {
     // Alert, // SMBUS mode only
 }
 
-// #[derive(Clone, Copy)]
-// #[repr(u8)]
-// /// Set master or slave mode.
-// pub enum I2cMode {
-//     Master = 0,
-//     Slave = 1,
-// }
-
 #[derive(Clone, Copy)]
 pub enum I2cDevice {
     One,
@@ -99,15 +92,100 @@ pub enum I2cDevice {
     Two,
     #[cfg(any(feature = "h7", feature = "wb"))]
     Three,
+    // Four ommitted for now; just need to add some non-standard
+    // periph clock setup/enable
+    // #[cfg(any(feature = "l5", feature = "h7"))]
+    // Four,
+}
+
+#[derive(Clone, Copy)]
+#[repr(u8)]
+/// Set master or slave mode. Sets the __ register, _ field.
+pub enum I2cMode {
+    /// In Master mode, the I2C interface initiates a data transfer and generates the clock signal. A
+    /// serial data transfer always begins with a START condition and ends with a STOP condition.
+    /// Both START and STOP conditions are generated in master mode by software.
+    Master = 0,
+    /// In Slave mode, the interface is capable of recognizing its own addresses (7 or 10-bit), and
+    /// the general call address. The general call address detection can be enabled or disabled by
+    /// software. The reserved SMBus addresses can also be enabled by software.
+    Slave = 1,
+}
+
+#[derive(Clone, Copy)]
+/// Set a preset I2C speed, based on RM tables: Examples of timings settings.
+/// Sets 5 fields of the TIMINGR register.
+pub enum I2cSpeed {
+    /// Standard-mode: 10kHz.
+    Standard10K,
+    /// Standard-mode: 100kHz.
+    Standard100K,
+    /// Fast-mode: 400kHz.
+    Fast400K,
+    /// Fast-mode +: 1Mhz.
+    FastPlus1M,
+}
+
+#[derive(Clone, Copy)]
+#[repr(u8)]
+/// Set the number of address bits to 7 or 10. Sets the CR2 register, ADD10 field.
+pub enum AddressBits {
+    B7 = 0,
+    B10 = 1,
+}
+
+#[derive(Clone, Copy, PartialEq)]
+// #[repr(u8)]
+/// Set the number of address bits to 7 or 10. Sets the CR1 register, ANFOFF and DNF fields.
+pub enum NoiseFilter {
+    /// Analog noise filter enabled.
+    Analog,
+    /// Digital filter enabled and filtering capability ( filters spikes with a length of)
+    /// up to (value) t_I2CCLK
+    Digital(u8),
+    /// Analog and digital filters disabled.
+    Disabled,
+}
+
+/// Initial configuration data for the I2C peripheral.
+pub struct I2cConfig {
+    /// Select master or slave mode. Defaults to Master.
+    pub mode: I2cMode,
+    // /// Select between standard, fast, and fast-plus speeds. Default to standard.
+    /// pub speed_mode: SpeedMode,
+    /// Select between one of 4 preset speeds. If you'd like to use custom
+    /// speed settings, use the PAC directly, with I2C disabled, after the
+    /// peripheral clocks are enabled by `new()`. Default to Standard mode, 100kHz.
+    pub speed: I2cSpeed,
+    /// Allows setting 7 or 10-bit addresses. Defaults to 7.
+    pub address_bits: AddressBits,
+    /// Select the analog noise filter, a digital filter, or no filter.
+    pub noise_filter: NoiseFilter,
+    /// Support for SMBUS, including hardware PEC, and alert pin. Defaults to false.
+    pub smbus: bool,
+    /// Optionally disable clock stretching. Defaults to false (stretching allowed)
+    /// Only relevant in slave mode.
+    pub nostretch: bool,
+}
+
+impl Default for I2cConfig {
+    fn default() -> Self {
+        Self {
+            mode: I2cMode::Master,
+            speed: I2cSpeed::Standard100K,
+            address_bits: AddressBits::B7,
+            noise_filter: NoiseFilter::Analog,
+            smbus: false,
+            nostretch: false,
+        }
+    }
 }
 
 /// Represents an Inter-Integrated Circuit (I2C) peripheral.
 pub struct I2c<R> {
-    regs: R,
+    pub regs: R,
     device: I2cDevice,
-    // mode: I2cMode,
-    /// SMBUS features like PEC enabled.
-    smbus: bool,
+    pub cfg: I2cConfig,
 }
 
 impl<R> I2c<R>
@@ -116,12 +194,7 @@ where
 {
     /// Initialize a I2C peripheral, including configuration register writes, and enabling and resetting
     /// its RCC peripheral clock. `freq` is in Hz.
-    pub fn new(
-        regs: R,
-        device: I2cDevice,
-        freq: u32, // todo: Set a division manually, like you do with SPI.
-        clocks: &Clocks,
-    ) -> Self {
+    pub fn new(regs: R, device: I2cDevice, cfg: I2cConfig, clocks: &Clocks) -> Self {
         free(|_| {
             let rcc = unsafe { &(*RCC::ptr()) };
 
@@ -136,89 +209,182 @@ where
                 #[cfg(any(feature = "h7", feature = "wb"))]
                 I2cDevice::Three => {
                     rcc_en_reset!(apb1, i2c3, rcc);
-                }
+                } // todo: APB1LR2 on L5, and AHB4 on H7. Fix it.
+                  // #[cfg(any(feature = "l5", feature = "h7"))]
+                  // I2cDevice::Four => {
+                  //     rcc_en_reset!(apb1, i2c4, rcc);
+                  // }
             }
         });
 
-        assert!(freq <= 1_000_000);
         // Make sure the I2C unit is disabled so we can configure it
         regs.cr1.modify(|_, w| w.pe().clear_bit());
 
-        // TODO review compliance with the timing requirements of I2C
-        // t_I2CCLK = 1 / PCLK1
-        // t_PRESC  = (PRESC + 1) * t_I2CCLK
-        // t_SCLL   = (SCLL + 1) * t_PRESC
-        // t_SCLH   = (SCLH + 1) * t_PRESC
-        //
-        // t_SYNC1 + t_SYNC2 > 4 * t_I2CCLK
-        // t_SCL ~= t_SYNC1 + t_SYNC2 + t_SCLL + t_SCLH
-        let i2cclk = clocks.apb1();
-        let ratio = i2cclk / freq - 4;
-        let (presc, scll, sclh, sdadel, scldel) = if freq >= 100_000 {
-            // fast-mode or fast-mode plus
-            // here we pick SCLL + 1 = 2 * (SCLH + 1)
-            let presc = ratio / 387;
+        // todo: Slave currently nonfunctional!
+        // todo: Check out the RM recipes for slave transmitter and receiver.
 
-            let sclh = ((ratio / (presc + 1)) - 3) / 3;
-            let scll = 2 * (sclh + 1) - 1;
+        // RM: I2C timings:
+        // The timings must be configured in order to guarantee a correct data hold and setup time,
+        // used in master and slave modes. This is done by programming the PRESC[3:0],
+        // SCLDEL[3:0] and SDADEL[3:0] bits in the I2C_TIMINGR register.
+        // ... Additionally, in master mode, the SCL clock high and low levels must be configured by
+        // programming the PRESC[3:0], SCLH[7:0] and SCLL[7:0] bits in the I2C_TIMINGR register
 
-            let (sdadel, scldel) = if freq > 400_000 {
-                // fast-mode plus
-                let sdadel = 0;
-                let scldel = i2cclk / 4_000_000 / (presc + 1) - 1;
+        // For these speed and frequency variables, we use the RM's conventions.
+        let t_i2cclk = clocks.apb1();
 
-                (sdadel, scldel)
-            } else {
-                // fast-mode
-                let sdadel = i2cclk / 8_000_000 / (presc + 1);
-                let scldel = i2cclk / 2_000_000 / (presc + 1) - 1;
+        // assert!(t_i2cclk < (t_low - f_f) / 4);
+        // assert!(t_i2cclk < t_high);
 
-                (sdadel, scldel)
-            };
+        // Set the prescaler using RM tables as a guide;
+        // L552 RM, Tables 324 - 326: Examples of timings settings.
+        // Note that the table only includes I2C clock multiples of 4Mhz (well, multiples of 8Mhz).
+        // In this case, we'll use the integer floor rounding to handle in-between
+        // values.
 
-            (presc, scll, sclh, sdadel, scldel)
-        } else {
-            // standard-mode
-            // here we pick SCLL = SCLH
-            let presc = ratio / 514;
-
-            let sclh = ((ratio / (presc + 1)) - 2) / 2;
-            let scll = sclh;
-
-            let sdadel = i2cclk / 2_000_000 / (presc + 1);
-            let scldel = i2cclk / 800_000 / (presc + 1) - 1;
-
-            (presc, scll, sclh, sdadel, scldel)
+        // We use this constant in several calculations.
+        let presc_const = match cfg.speed {
+            I2cSpeed::Standard10K => 4_000_000,
+            I2cSpeed::Standard100K => 4_000_000,
+            I2cSpeed::Fast400K => 8_000_000,
+            // Note: The 16Mhz example uses F+ / 16. The other 2 examples
+            // use 8e6.
+            I2cSpeed::FastPlus1M => 8_000_000,
         };
 
-        let presc = u8(presc).unwrap();
-        assert!(presc < 16);
-        let scldel = u8(scldel).unwrap();
-        assert!(scldel < 16);
-        let sdadel = u8(sdadel).unwrap();
-        assert!(sdadel < 16);
-        let sclh = u8(sclh).unwrap();
-        let scll = u8(scll).unwrap();
+        // This is (offset by 1) which we set as the prescaler.
+        let presc_val = t_i2cclk / presc_const;
+
+        // Hit the target freq by setting up t_scll (Period of SCL low)
+        // to be half the whole period. These constants
+        // are from the tables.
+        let freq = match cfg.speed {
+            I2cSpeed::Standard10K => 10_000,
+            I2cSpeed::Standard100K => 100_000,
+            I2cSpeed::Fast400K => 400_000,
+            I2cSpeed::FastPlus1M => 1_000_000,
+        };
+
+        // Set SCLL (SCL low time) to be half the duty period
+        // associated with the target frequency.
+        let scll_val = presc_const / (2 * freq);
+
+        // SCLH is smaller than SCLH. For standard mode it's close, although
+        // in the example tables, 20% different for 100Khz, and 2% different for
+        // 10K. THis may be due to delays
+        // involved. The ratio is different for Fast-mode and Fast-mode+.
+        // todo: Come back to this. How should we set this?
+        let sclh_val = match cfg.speed {
+            I2cSpeed::Standard10K => scll_val - 4,
+            I2cSpeed::Standard100K => scll_val - 4,
+            I2cSpeed::Fast400K => scll_val * 4 / 10,
+            I2cSpeed::FastPlus1M => scll_val / 2,
+        };
+
+        // Timing prescaler. This field is used to prescale I2CCLK in order to generate the clock period tPRESC used for
+        // data setup and hold counters (refer to I2C timings on page 1495) and for SCL high and low
+        // level counters (refer to I2C master initialization on page 1510).
+        // Sets TIMINGR reg, PRESC field.
+
+        let presc = presc_val - 1;
+
+        // SCL low period (master mode)
+        // This field is used to generate the SCL low period in master mode.
+        // tSCLL = (SCLL+1) x tPRESC
+        // Note: SCLL is also used to generate tBUF and tSU:STA timings.
+        // Sets TIMINGR reg, SCLL field.
+        let scll = scll_val - 1;
+
+        // SCL high period (master mode)
+        // This field is used to generate the SCL high period in master mode.
+        // tSCLH = (SCLH+1) x tPRESC
+        // Note: SCLH is also used to generate tSU:STO and tHD:STA timing
+        // Set the clock prescaler value. Sets TIMINGR reg, SCLH field.
+        let sclh = sclh_val - 1;
+
+        // todo: Can't find the sdadel and scldel pattern
+        // Data hold time
+        // This field is used to generate the delay tSDADEL between SCL falling edge and SDA edge. In
+        // master mode and in slave mode with NOSTRETCH = 0, the SCL line is stretched low during
+        // tSDADEL.
+        // tSDADEL= SDADEL x tPRESC
+        // Note: SDADEL is used to generate tHD:DAT timing
+        // Sets TIMINGR reg, SDADEL field.
+        let sdadel = match cfg.speed {
+            I2cSpeed::Standard10K => 0x2,
+            I2cSpeed::Standard100K => 0x2,
+            I2cSpeed::Fast400K => 0x3,
+            I2cSpeed::FastPlus1M => 0x0,
+        };
+
+        // Data setup time
+        // This field is used to generate a delay tSCLDEL between SDA edge and SCL rising edge. In
+        // master mode and in slave mode with NOSTRETCH = 0, the SCL line is stretched low during
+        // tSCLDEL.
+        // tSCLDEL = (SCLDEL+1) x tPRESC
+        // Note: tSCLDEL is used to generate tSU:DAT timing
+        // Sets TIMINGR reg, SCLDEL field.
+        let scldel = match cfg.speed {
+            I2cSpeed::Standard10K => 0x4,
+            I2cSpeed::Standard100K => 0x4,
+            I2cSpeed::Fast400K => 0x3,
+            I2cSpeed::FastPlus1M => 0x1,
+        };
+
+        // The fields for PRESC, SCLDEL, and SDADEL are 4-bits; don't overflow.
+        // The other TIMINGR fields we set are 8-bits, so won't overflow with u8.
+        assert!(presc <= 15);
+        assert!(scldel <= 15);
+        assert!(sdadel <= 15);
+
+        assert!(scll <= 255);
+        assert!(sclh <= 255);
 
-        // Configure for "fast mode" (400 KHz)
-        // RM: Before enabling the peripheral, the I2C master clock must be configured by setting the
-        // SCLH and SCLL bits in the I2C_TIMINGR register.
         regs.timingr.write(|w| unsafe {
-            w.presc().bits(presc);
-            w.scll().bits(scll);
-            w.sclh().bits(sclh);
-            w.sdadel().bits(sdadel);
-            w.scldel().bits(scldel)
+            w.presc().bits(presc as u8);
+            w.scldel().bits(scldel as u8);
+            w.sdadel().bits(sdadel as u8);
+            w.sclh().bits(sclh as u8);
+            w.scll().bits(scll as u8)
         });
 
-        // Enable the peripheral
-        regs.cr1.write(|w| w.pe().set_bit());
+        // Before enabling the I2C peripheral by setting the PE bit in I2C_CR1 register, the user must
+        // configure the noise filters, if needed. By default, an analog noise filter is present on the SDA
+        // and SCL inputs. This analog filter is compliant with the I2C specification which requires the
+        // suppression of spikes with a pulse width up to 50 ns in Fast-mode and Fast-mode Plus. The
+        // user can disable this analog filter by setting the ANFOFF bit, and/or select a digital filter by
+        // configuring the DNF[3:0] bit in the I2C_CR1 register.
+        // When the digital filter is enabled, the level of the SCL or the SDA line is internally changed
+        // only if it remains stable for more than DNF x I2CCLK periods. This allows spikes with a
+        // programmable length of 1 to 15 I2CCLK periods to be suppressed.
+        let (anf_bit, dnf_bits) = match cfg.noise_filter {
+            NoiseFilter::Analog => (false, 0),
+            NoiseFilter::Digital(filtering_len) => {
+                assert!(filtering_len <= 0b1111);
+                (true, filtering_len)
+            }
+            NoiseFilter::Disabled => (true, 0),
+        };
 
-        Self {
-            regs,
-            device,
-            smbus: false,
+        regs.cr1.modify(|_, w| unsafe {
+            w.anfoff().bit(anf_bit);
+            w.dnf().bits(dnf_bits)
+        });
+
+        if let I2cMode::Slave = cfg.mode {
+            regs.cr1.modify(|_, w| w.nostretch().bit(cfg.nostretch));
         }
+
+        let mut result = Self { regs, device, cfg };
+
+        if result.cfg.smbus {
+            result.enable_smbus();
+        }
+
+        // Enable the peripheral
+        result.regs.cr1.write(|w| w.pe().set_bit());
+
+        result
     }
 
     /// Enable SMBus support. See L44 RM, section 37.4.11: SMBus initialization
@@ -244,6 +410,9 @@ where
 
         // todo: Timeout detection?
 
+        // todo: HWCFGR Missing from PAC
+        // self.regs.hwcfgr.modify(|_, w| w.smbus().set_bit());
+
         if originally_enabled {
             self.regs.cr1.modify(|_, w| w.pe().set_bit());
         }
@@ -263,7 +432,7 @@ where
 
         for byte in bytes {
             // Wait until we have received something
-            busy_wait!(self.regs, rxne, bit_is_set);
+            busy_wait!(self.regs, rxne);
 
             *byte = self.regs.rxdr.read().rxdata().bits();
         }
@@ -284,7 +453,7 @@ where
             // Wait until we are allowed to send data
             // (START has been ACKed or last byte when
             // through)
-            busy_wait!(self.regs, txis, bit_is_set); // TXDR register is empty
+            busy_wait!(self.regs, txis); // TXDR register is empty
 
             // Put byte on the wire
             self.regs.txdr.write(|w| unsafe { w.txdata().bits(*byte) });
@@ -306,14 +475,14 @@ where
             // Wait until we are allowed to send data
             // (START has been ACKed or last byte went through)
 
-            busy_wait!(self.regs, txis, bit_is_set); // TXDR register is empty
+            busy_wait!(self.regs, txis); // TXDR register is empty
 
             // Put byte on the wire
             self.regs.txdr.write(|w| unsafe { w.txdata().bits(*byte) });
         }
 
         // Wait until the write finishes before beginning to read.
-        busy_wait!(self.regs, tc, bit_is_set); // transfer is complete
+        busy_wait!(self.regs, tc); // transfer is complete
 
         // reSTART and prepare to receive bytes into `buffer`
 
@@ -321,7 +490,7 @@ where
 
         for byte in buffer {
             // Wait until we have received something
-            busy_wait!(self.regs, rxne, bit_is_set);
+            busy_wait!(self.regs, rxne);
 
             *byte = self.regs.rxdr.read().rxdata().bits();
         }
@@ -337,7 +506,7 @@ where
         self.regs.cr2.write(|w| {
             unsafe {
                 // Addressing mode (7-bit or 10-bit): ADD10
-                w.add10().clear_bit();
+                w.add10().bit(self.cfg.address_bits as u8 != 0);
                 // Slave address to be sent: SADD[9:0]
                 // SADD0: "This bit is don’t care"
                 // SADD[7:1]: "These bits should be written with the 7-bit slave address to be sent"
@@ -357,7 +526,7 @@ where
                                           // If the SMBus master wants to send a STOP condition after the PEC, automatic end mode
                                           // must be selected (AUTOEND=1). In this case, the STOP condition automatically follows the
                                           // PEC transmission.
-                w.pecbyte().bit(self.smbus);
+                w.pecbyte().bit(self.cfg.smbus);
                 w.start().set_bit()
             }
         });
@@ -372,7 +541,7 @@ where
     fn set_cr2_read(&mut self, addr: u8, len: u8) {
         self.regs.cr2.write(|w| {
             unsafe {
-                w.add10().clear_bit();
+                w.add10().bit(self.cfg.address_bits as u8 != 0);
                 w.sadd().bits(u16(addr << 1));
                 w.rd_wrn().set_bit(); // read
                 w.nbytes().bits(len);
@@ -383,7 +552,7 @@ where
                                        // after NBYTES-1 data have been received, the next received byte is automatically checked
                                        // versus the I2C_PECR register content. A NACK response is given to the PEC byte, followed
                                        // by a STOP condition.
-                w.pecbyte().bit(self.smbus);
+                w.pecbyte().bit(self.cfg.smbus);
                 w.start().set_bit()
             }
         });
@@ -441,18 +610,6 @@ where
         // (The steps above are handled in the write this function performs.)
         self.set_cr2_write(addr, len as u8, autoend);
 
-        // todo: not usgin set_cr2_write to ts due to auto ending?
-        // self.regs.cr2.write(|w| {
-        //     unsafe {
-        //         w.add10().clear_bit();
-        //         w.sadd().bits(u16(addr << 1));
-        //         w.rd_wrn().clear_bit(); // write
-        //         w.nbytes().bits(len as u8);
-        //         w.autoend().set_bit(); // software end mode
-        //         w.start().set_bit()
-        //     }
-        // });
-
         // • In slave mode:
         // – With NOSTRETCH=0, when all data are transferred using DMA, the DMA must be
         // initialized before the address match event, or in ADDR interrupt subroutine, before
